use camino::Utf8PathBuf;
use serde::{Deserialize, Serialize};
use xmpp::BareJid;

use crate::xdg::xdg_config;

#[derive(Clone, Debug, Default, Serialize, Deserialize)]
pub struct ConfigFile {
    pub accounts: Vec<AccountConfig>,
}

#[derive(Clone, Debug, Serialize, Deserialize)]
pub struct AccountConfig {
    jid: BareJid,
    password: String,
}

impl AccountConfig {
    pub fn new(jid: &BareJid, password: &str) -> Self {
        Self {
            jid: jid.clone(),
            password: password.to_string(),
        }
    }
}

impl ConfigFile {
    pub fn xdg_path() -> Utf8PathBuf {
        xdg_config("rino").join("config.toml")
    }

    pub fn from_xdg() -> Self {
        let config_path = Self::xdg_path();
        if config_path.is_file() {
            let content = std::fs::read_to_string(&config_path).unwrap();
            toml::from_str(&content).unwrap()
        } else {
            Self::default()
        }
    }

    pub fn to_xdg(&self) {
        let config_path = Self::xdg_path();
        let config_dir = config_path.parent().unwrap();
        if !config_dir.is_dir() {
            std::fs::create_dir_all(&config_dir).unwrap();
        }
        std::fs::write(&config_path, toml::to_string(&self).unwrap()).unwrap()
    }

    pub fn get_account_password(&self, jid: &BareJid) -> Option<&str> {
        self.accounts
            .iter()
            .find(|acc| &acc.jid == jid)
            .map(|x| x.password.as_str())
    }

    pub fn save_account(&mut self, jid: &BareJid, password: &str) {
        if let Some(p) = self.get_account_password(jid) {
            if p == password {
                return;
            }

            for acc in self.accounts.iter_mut() {
                if &acc.jid == jid {
                    acc.password = password.to_string();
                }
            }
        } else {
            let account = AccountConfig::new(jid, password);
            self.accounts.push(account);
        }
        self.to_xdg();
    }

    pub fn get_first_account(&self) -> Option<(&BareJid, &String)> {
        for account in &self.accounts {
            return Some((&account.jid, &account.password));
        }

        return None;
    }
}
